# (c) 2023 Christophe Gueneau
from time import sleep

class Relais_4_grove():
    """Classe définissant le relais 4 canaux de Grove"""
    def __init__(self, i2c):
        self._i2c = i2c
        self._adr_i2c = 0x11                        # Adresse I2C par défaut
        self._adr_reg_ctrl = 0x10                   # Registre d'adresse 0x10 contient l'état des relais
        self._adr_reg_i2c_ecriture = 0x11           # Registre d'adresse 0x11 permet de modifier l'adressse I2C de 0x00 ~ 0x7F
        self._adr_reg_i2c_lecture = 0x12            # Registre d'adresse 0x12 contient l'adressse I2C courante
        self._adr_reg_firmware_ver = 0x13           # Registre d'adresse 0x13 contien la version du firmware
        self._etat_relais = 0

    def adresse_i2c(self, adresse):
        """adresse I2C si elle a été changée"""
        self._adr_i2c = adresse

    def change_i2c(self, ancienne_adresse, nouvelle_adresse):
        self._ecriture_registre(ancienne_adresse, self._adr_reg_i2c_ecriture, nouvelle_adresse)
        self._adr_i2c = nouvelle_adresse
        sleep(1)

    def scan_i2c_adresse(self):
        adresses = self._i2c.scan()
        if len(adresses) == 0:
            print("Pas d'adresses trouvees")
        else:
            print(len(adresses), "adresses I2C trouves")
            print("Decimal | Hexadecimal")
            for adresses in adresses:
                print("  ", adresses, "  |  ", hex(adresses))

    def _ecriture_registre(self, adresse_i2c, adresse_registre, val):
        buf = bytes((adresse_registre, val))
        self._i2c.writeto(adresse_i2c, buf)

    def etat(self):
        return self._etat_relais

    def ctrl_global(self, etat):
        """
        controle des 4 relais
        un octet représente l'état des 4 relais
        relais: 4 3 2 1
        état: 0b0000 -> 0x00 = 0   (tous off)
        état: 0b1111 -> 0x0f = 15  (tous on)
        """
        self._etat_relais = etat
        self._ecriture_registre(self._adr_i2c, self._adr_reg_ctrl, self._etat_relais)

    def ctrl(self, relais, etat):
        if etat == 1:
            self._etat_relais |= (1 << (relais - 1))
        else:
            self._etat_relais &= ~(1 << (relais - 1))
        self._ecriture_registre(self._adr_i2c, self._adr_reg_ctrl, self._etat_relais)


if __name__ == '__main__':
    
    from machine import Pin, I2C
    from time import sleep

    # Crée un objet I2C
    i2c0 = I2C(0, scl=Pin(9), sda=Pin(8), freq=400000)    # I2C0 utilisé  

    # Crée un objet Relais_4_grove
    relais = Relais_4_grove(i2c0)

    relais.adresse_i2c(0x11)                              # Adresse I2C, 0x11 par defaut 

    relais.scan_i2c_adresse()                             # Scanne les adresses I2C
    
    #relais.change_i2c(0x11,0x20)                         # Changement d'adresse I2C
    
    relais.ctrl_global(0b0100)                            # Marche relais 3, arrêt des autres
    sleep(2)
    relais.ctrl_global(0b1100)                            # Marche relais 3 et 4, arrêt des autres
    sleep(2)

    relais.ctrl(2,1)                                      # Marche relais 2
    sleep(2)
    relais.ctrl(2,0)                                      # Arrêt relais 2

    print("Etat des relais : ", relais.etat())            # Retourne un octet représentant l'état des 4 relais

   